Scriptname ImWaMaintenance extends ReferenceAlias 


GlobalVariable Property ImWaChargeRateGV Auto
GlobalVariable Property ImWaWardAbsorbPerkGV Auto
GlobalVariable Property ImWaArrowsDmgReductionGV Auto
GlobalVariable Property ImWaWardCostMultGV Auto
GlobalVariable Property ImWaWardMagnitudeMultGV Auto
GlobalVariable Property ImWaSurroundingMasterWardMeleeProtectionGV Auto

Quest Property MCMQuest Auto

Perk Property wardAbsorb Auto
Perk Property ImWaArrowsDamageRedutionPlayerPerk Auto
Perk Property ImWaCostMultPerk Auto
Perk Property ImWaMasterWardDamageRedutionPerk Auto

formlist Property wardsSpellList Auto
float[] Property vanillaMagnitudeArray Auto

formlist Property ImWaHandledShoutsFormList Auto
formlist Property ImWaAdditionalShoutsFormList Auto

Spell Property ImWaVoiceUnrelentingForce1 Auto

GlobalVariable Property ImWaVersion Auto
GlobalVariable Property ImWaInstalledVersion Auto
Message Property ImWaInitializeMsg Auto

Bool Property debugVerbosity = False Auto




;------Events------

Event OnInit()
	Debug.TraceConditional(self + " OnInit event", debugVerbosity)
	Actor playerRef = Game.GetPlayer()
	If ! playerRef.HasPerk(ImWaCostMultPerk)
		Game.Getplayer().AddPerk(ImWaCostMultPerk)
	EndIf
	If ! playerRef.HasPerk(ImWaArrowsDamageRedutionPlayerPerk)
		Game.Getplayer().AddPerk(ImWaArrowsDamageRedutionPlayerPerk)
	EndIf
	PerksandSettingsUpdate()
	MagnitudeUpdate()
	ShoutsUpdate(ImWaHandledShoutsFormList)
	ShoutsUpdate(ImWaAdditionalShoutsFormList)
	MCMQuest.Stop()
	MCMQuest.Start()
	ImWaInstalledVersion.SetValue(ImWaVersion.GetValue())
	ImWaInitializeMsg.Show()
EndEvent


Event OnPlayerLoadGame()
	Debug.TraceConditional(self + " OnPlayerLoad event", debugVerbosity)
	If ImWaVersion.GetValue() != ImWaInstalledVersion.GetValue()
		Debug.TraceConditional(self + " Update detected", debugVerbosity)
		self.GetOwningQuest().Stop()
		self.GetOwningQuest().Start()
	EndIf
	PerksandSettingsUpdate()
	MagnitudeUpdate()
	ShoutsUpdate(ImWaHandledShoutsFormList)
	ShoutsUpdate(ImWaAdditionalShoutsFormList)
EndEvent




;------Functions------

Function MagnitudeUpdate()
	Debug.TraceConditional(Self + " Starting magnitude update.", debugVerbosity)
	float magnitudeMult = ImWaWardMagnitudeMultGV.GetValue()
	int index = 0
	int numberOfSpells = wardsSpellList.GetSize()
	While index < numberOfSpells
		Spell currentSpell = wardsSpellList.GetAt(index) as Spell
		Debug.TraceConditional(self + " Updating spell:" + currentSpell, debugVerbosity)
		int effectIndex = 0
		int numberOfEffects = currentSpell.GetNumEffects()
		While effectIndex < numberOfEffects
			float magnitude =  vanillaMagnitudeArray[index] * magnitudeMult
			currentSpell.SetNthEffectMagnitude(effectIndex, magnitude)
			Debug.TraceConditional(Self + " Magnitude set on effect " + currentSpell.GetNthEffectMagicEffect(effectIndex) + " New Magnitude:" + currentSpell.GetNthEffectMagnitude(effectIndex), debugVerbosity)
			effectIndex = effectIndex + 1
		EndWhile
		index = index + 1
		Debug.TraceConditional(Self + " New cost:" + currentSpell.GetMagickaCost() + "\n\n", debugVerbosity)
	EndWhile
EndFunction

Function ShoutsUpdate(Formlist shoutList) 
	Debug.TraceConditional(Self + " Starting shouts update at " + Utility.GetCurrentRealTime(), debugVerbosity)
	int index = 0
	int numberOfSpells = shoutList.GetSize()
	EquipSlot voice = ImWaVoiceUnrelentingForce1.GetEquipType()
	While index < numberOfSpells
		Spell currentShout = shoutList.GetAt(index) as Spell
		If currentShout.GetEquipType() != voice
			currentShout.SetEquipType(voice)
			Debug.TraceConditional(Self + currentShout as string + " set to equip type voice" , debugVerbosity)
		EndIf
		index = index + 1
	Endwhile
	Debug.TraceConditional(Self + " Shouts update finished " + Utility.GetCurrentRealTime(), debugVerbosity)
EndFunction

Function PerksandSettingsUpdate()
	Game.SetGameSettingFloat("fMagicWardPowerMaxBase", ImWaChargeRateGV.GetValue())
	If wardAbsorb.GetNumEntries() == 1
		wardAbsorb.SetNthEntryValue(0, 0, ImWaWardAbsorbPerkGV.GetValue())
	Else
		Debug.TraceConditional(self + " other mod modifies the Ward Absorb perk, value update aborted.", debugVerbosity)
	EndIf
	ImWaArrowsDamageRedutionPlayerPerk.SetNthEntryValue(0, 0, ImWaArrowsDmgReductionGV.GetValue())
	ImWaCostMultPerk.SetNthEntryValue(0, 0, ImWaWardCostMultGV.GetValue())
	ImWaMasterWardDamageRedutionPerk.SetNthEntryValue(0, 0, 1.0 - (ImWaSurroundingMasterWardMeleeProtectionGV.GetValue() / 100.0 ))
EndFunction










