Scriptname ImWaWardEffectsScript extends activeMagicEffect
{Wards Functionalities Extended scripted ward effects.}


actor wardCasterActor ;actor casting the ward

sound Property wardHitsound Auto
sound Property wardBreaksound Auto
sound Property masterWardLoopSound Auto
int masterWardSoundID

formList Property handledShoutList Auto ;List of shouts (as spell form) handled by the script as shouts.
formList Property correspondingShoutList Auto
formList Property ImWaAdditionalShoutsFormList Auto ;Non-vanilla shouts for mod support
FormList Property ImWaExcludedShoutsFormList Auto

spell Property masterWardSpell Auto
magicEffect Property masterWardMagicEffect Auto
Perk Property ImWaMasterWardDamageRedutionPerk Auto

spell Property ImWaVoiceUnrelentingForce1 Auto
spell Property VoiceUnrelentingForce3 Auto
spell Property DLC2VoiceCyclone01 Auto
spell Property DLC2VoiceCyclone02 Auto
spell Property DLC2VoiceCyclone03 Auto

int Property pushForce = 15 Auto
int Property cycloneForce = 10 Auto

spell Property staggerOnWardBreakSpell Auto

Keyword Property ImmuneParalysis  Auto

float Property wardBlockingAngle Auto
int Property maximumSkillLevel Auto

globalVariable Property ImWarangedWardpowerDamageMultGV Auto ;User MCM customization options
globalVariable Property ImWameleeWardpowerDamageMultGV Auto
globalVariable Property ImWaVoiceBreaksWardsToggleGV Auto
globalVariable Property ImWaWardBlocksWeaponsToggleGV Auto
globalVariable Property ImWaShoutWardpowerDamageGV Auto
globalVariable Property ImWaShoutsOnWardsAltMethodGV Auto

Activator Property FxEmptyActivator Auto

bool Property debugVerbosity Auto
bool Property lockWardHitEvent Auto
bool Property abortHitWhenBusy Auto
bool onHitFunctionBusy = False
bool onWardHitFunctionBusy = False


int hitCount = 0



Event OnEffectStart(actor akTarget, actor akCaster)
	wardCasterActor = akCaster
	If wardBlockingAngle == 180
		masterWardSoundID = masterWardLoopSound.play(wardCasterActor)
		If ! wardCasterActor.HasPerk(ImWaMasterWardDamageRedutionPerk)
			wardCasterActor.AddPerk(ImWaMasterWardDamageRedutionPerk)
		EndIf
	ElseIf wardCasterActor.HasMagicEffect(masterWardMagicEffect)
		wardCasterActor.DispelSpell(masterWardSpell)
	EndIf
	If ImWaWardBlocksWeaponsToggleGV.GetValue() == 1 && wardBlockingAngle != 180
		wardCasterActor.SetAnimationVariablebool("IsBlocking", true)
	EndIf
	GotoState("WardCanBeHit")
EndEvent

Event OnEffectFinish(actor akTarget, actor akCaster)
	If akCaster.GetAnimationVariablebool("IsBlocking")
		akCaster.SetAnimationVariablebool("IsBlocking", false)
	EndIf
	If wardBlockingAngle == 180
		Sound.StopInstance(masterWardSoundID) 
		If !akCaster.HasMagicEffect(masterWardMagicEffect) && akCaster.HasPerk(ImWaMasterWardDamageRedutionPerk)
			akCaster.RemovePerk(ImWaMasterWardDamageRedutionPerk)
		EndIf
	Endif
	If akCaster.GetAV("wardpower") < 0
		akCaster.SetAV("Wardpower", 0)
		akCaster.ModAV("Wardpower", 0)
	EndIf
	GotoState("")
EndEvent
	
	
	
	
float Function EstimateWardDamage(actor weaponWielder, int weaponBaseDamage, int weaponType, bool powerAttack, bool rangeWeaponIsEnchanted)
;Estimates damage to apply to Wardpower, takes into account skill, base weapon damage, power attack, potions and enchantments. Does not take into account perks and weapon tempering.
	float minimumSkillMultiplier = 1
	float maximumSkillMultiplier = 1
	if weaponWielder == Game.GetPlayer()
		minimumSkillMultiplier = Game.GetGameSettingFloat("fDamagePCSkillMin")
		maximumSkillMultiplier = Game.GetGameSettingFloat("fDamagePCSkillMax")
	Else
		minimumSkillMultiplier = Game.GetGameSettingFloat("fDamageSkillMin")
		maximumSkillMultiplier = Game.GetGameSettingFloat("fDamageSkillMax")
	EndIf
	float userMultiplier
	float wielderSkillLevel
	float estimatedWardDamage
	float skillDamageMultiplier
	float avMod
	float avPowerMod
	If weaponType != 0
		If weaponType <= 4 ;One handed weapons
			wielderSkillLevel = weaponWielder.GetAV("OneHanded")
			avMod = (weaponWielder.GetAV("OneHandedMod") / 100.0) + 1.0
			avPowerMod = (weaponWielder.GetAV("OneHandedPowerMod") / 100.0) + 1.0
			userMultiplier = ImWameleeWardpowerDamageMultGV.GetValue()
		ElseIf weaponType == 5 || weaponType == 6 ;Two handed weapons
			wielderSkillLevel = weaponWielder.GetAV("TwoHanded")
			avMod = (weaponWielder.GetAV("TwoHandedMod") / 100.0) + 1.0
			avPowerMod = (weaponWielder.GetAV("TwoHandedPowerMod") / 100.0) + 1.0
			userMultiplier = ImWameleeWardpowerDamageMultGV.GetValue()
		ElseIf weaponType == 7 || weaponType == 9 ;Ranged weapons
			wielderSkillLevel = weaponWielder.GetAV("Marksman")
			avMod = (weaponWielder.GetAV("MarksmanMod") / 100.0) + 1.0
			avPowerMod = (weaponWielder.GetAV("MarksmanPowerMod") / 100.0) + 1.0
			userMultiplier = ImWarangedWardpowerDamageMultGV.GetValue()
			If rangeWeaponIsEnchanted ;this is to balance ward damage because enchantments from ranged weapons already does ward damage in vanilla
				userMultiplier = userMultiplier / 2
			Endif
		EndIf
		skillDamageMultiplier = minimumSkillMultiplier + ((maximumSkillMultiplier - minimumSkillMultiplier) * wielderSkillLevel / maximumSkillLevel)
		estimatedWardDamage = weaponBaseDamage * userMultiplier * skillDamageMultiplier * avMod * avPowerMod
		Debug.TraceConditional(self + "Base weapon dmg:" + weaponBaseDamage + " User mult:" + userMultiplier + " Skill mult:" + skillDamageMultiplier + " AVMod:" + avMod + " AVPowerMod:" + avPowerMod, debugVerbosity)
	Else ;Unarmed or animal attacks.
		userMultiplier = ImWameleeWardpowerDamageMultGV.GetValue()
		estimatedWardDamage = weaponWielder.GetAV("UnarmedDamage") * userMultiplier
		Debug.TraceConditional(self + "Unarmed or animal attack. UnarmedDamage:" + estimatedWardDamage, debugVerbosity)
	EndIf
	If powerAttack
		estimatedWardDamage = estimatedWardDamage * (1 + Game.GetGameSettingFloat("fPowerAttackDefaultBonus"))
	EndIf
	If estimatedWardDamage < 0
		estimatedWardDamage = 0
	EndIf
	Return estimatedWardDamage
EndFunction


Bool Function ApplyWardDamage(float currentWardPower, float damage, ObjectReference Aggressor)
	bool wardBroken
	float calculatedWardPower
	calculatedWardPower = currentWardPower - damage
	Debug.TraceConditional(self + "currentWardPower:" + currentWardPower + " calculatedWardPower:" + calculatedWardPower, debugVerbosity)
	If calculatedWardPower <= 0 && currentWardPower > 0
		wardBroken = True
		wardCasterActor.DamageAV("Wardpower", damage)
		wardBreaksound.Play(wardCasterActor)
		staggerOnWardBreakSpell.cast(Aggressor, wardCasterActor)
		If wardBlockingAngle == 180 ;need to manually dispel Master Ward.
			wardCasterActor.DispelSpell(masterWardSpell)
			Debug.TraceConditional(self + "Dispelling master ward.", debugVerbosity)
		EndIf
	ElseIf currentWardPower > 0
		wardBroken = False
		wardCasterActor.DamageAV("Wardpower", damage)
	EndIf
	Return wardBroken
EndFunction




State WardCanBeHit

	Event OnHit(ObjectReference akAggressor, form akSource, projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
	
		If onHitFunctionBusy && abortHitWhenBusy
			Debug.TraceConditional(self + "Hit event aborted", debugVerbosity)
			Return
		ElseIf akAggressor && akSource
			onHitFunctionBusy = True
			
			float hitAngle = wardCasterActor.GetHeadingAngle(akAggressor)
			Debug.TraceConditional(self + "Hit detected at angle " + hitAngle + " Time:" + Utility.GetCurrentRealTime(), debugVerbosity)
			
			float currentWardPower
			currentWardPower = wardCasterActor.GetAV("Wardpower")
			
			If (ImWaWardBlocksWeaponsToggleGV.GetValue() == 1) && (currentWardPower > 0) && (-wardBlockingAngle <= hitAngle && hitAngle <= wardBlockingAngle) && (akSource as weapon)
				wardHitsound.Play(wardCasterActor)
				Debug.TraceConditional(self + "Hit Sound played at time: " + Utility.GetCurrentRealTime(), debugVerbosity)
				weapon weaponUsed = (akSource as weapon)
				int weaponType = weaponUsed.GetweaponType()
				Debug.TraceConditional(self + "Weapon hit ward." + " Aggressor:" + akAggressor + " Source: " + akSource + " Projectile:" + akProjectile + " weaponType:" + weaponType, debugVerbosity)
				
				If weaponType <= 6
					float weaponDamage = EstimateWardDamage(akAggressor as actor, weaponUsed.GetBaseDamage(), weaponType, abPowerAttack, False)
					ApplyWardDamage(currentWardPower, weaponDamage, akAggressor)
					Debug.TraceConditional(self as string + weaponDamage + " melee weapon damage applied to wardpower.", debugVerbosity)
				ElseIf weaponType == 7 || weaponType == 9
					If (weaponUsed.GetEnchantment() as enchantment)
						float weaponDamage = EstimateWardDamage(akAggressor as actor, weaponUsed.GetBaseDamage(), weaponType, abPowerAttack, True)
						ApplyWardDamage(currentWardPower, weaponDamage, akAggressor)
						Debug.TraceConditional(self + "Bow/crossbow is enchanted. Enchantment:" + weaponUsed.GetEnchantment() , debugVerbosity)
						Debug.TraceConditional(self as string + weaponDamage + " Bow/crossbow weapon damage applied to wardpower.", debugVerbosity)
					Else
						float weaponDamage = EstimateWardDamage(akAggressor as actor, weaponUsed.GetBaseDamage(), weaponType, abPowerAttack, False)
						ApplyWardDamage(currentWardPower, weaponDamage, akAggressor)
						Debug.TraceConditional(self as string + weaponDamage + " Bow/crossbow weapon damage applied to wardpower.", debugVerbosity)
					EndIf
				EndIf
			EndIf
			
			If wardCasterActor.GetAV("Wardpower") < 0
				wardCasterActor.SetAV("Wardpower", 0)
				wardCasterActor.ModAV("Wardpower", 0)
			EndIf
			
			Debug.TraceConditional(self + "End of OnHit event." + " Time:" + Utility.GetCurrentRealTime() + "\n\n", debugVerbosity)
			
		EndIf
		onHitFunctionBusy = False
	EndEvent
	
	
	Event OnWardHit(ObjectReference akCaster, Spell akSpell, int aiStatus)
	
		While onWardHitFunctionBusy && lockWardHitEvent ;Allows to treat events one at the time, makes clearer logs but delays effects.
			Utility.Wait(0.1)
			;Debug.TraceConditional(self + "Additional OnWardHit event waiting.", debugVerbosity)	
		Endwhile
		onWardHitFunctionBusy = True
		Debug.TraceConditional(self as string + wardCasterActor + " ward was hit by " + akSpell + " from actor " + akCaster + " aiStatus:" + aiStatus + " Time:" + Utility.GetCurrentRealTime(), debugVerbosity)
		
		If (ImWaVoiceBreaksWardsToggleGV.GetValue() == 1) && akSpell && akCaster && akSpell.GetEquipType() == ImWaVoiceUnrelentingForce1.GetEquipType()
			float currentWardPower = wardCasterActor.GetAV("Wardpower")
			If ImWaShoutsOnWardsAltMethodGV.GetValue() == 1.0
				If !ImWaExcludedShoutsFormList.HasForm(akSpell) && ApplyWardDamage(currentWardPower, ImWaShoutWardpowerDamageGV.GetValue(), akCaster)
					ObjectReference castingObjectRef = wardCasterActor.PlaceAtMe(FxEmptyActivator)
					castingObjectRef.MoveTo(wardCasterActor, 0.0, 0.0, wardCasterActor.GetHeight())
					akSpell.Cast(castingObjectRef, wardCasterActor)
					Debug.Trace(self + "New method, shout casted" + "Time:" + Utility.GetCurrentRealTime())
					If akSpell == VoiceUnrelentingForce3
						akCaster.PushActorAway(wardCasterActor, pushForce)
					ElseIf wardCasterActor.HasKeyword(ImmuneParalysis) == false && (akSpell == DLC2VoiceCyclone01 || akSpell == DLC2VoiceCyclone02 || akSpell == DLC2VoiceCyclone03)
						wardCasterActor.PushActorAway(wardCasterActor, cycloneForce)
					EndIf
					castingObjectRef.InterruptCast()
					Utility.Wait(0.1)
					castingObjectRef.Delete()
				EndIf
			Else
				int shoutIndex = handledShoutList.Find(akSpell)
				If shoutIndex != -1 ;handles vanilla shouts, apply ward damage and cast copy shout if broken
					Debug.TraceConditional(self + "Old method, handled shout hit ward, found at index:" + shoutIndex, debugVerbosity)
					If ApplyWardDamage(currentWardPower, ImWaShoutWardpowerDamageGV.GetValue(), akCaster)
						spell shoutToCast = (correspondingShoutList.GetAt(shoutIndex) as spell)
						Debug.TraceConditional(self + "Ward broken by shout, casting:" + shoutToCast, debugVerbosity)
						shoutToCast.cast(akCaster, wardCasterActor)
					EndIf
				Else ;support for non-vanilla shouts, applies ward damage
					shoutIndex = ImWaAdditionalShoutsFormList.Find(akSpell)
					If shoutIndex != -1
						Debug.TraceConditional(self + "Old method, other shout hit ward, found at index:" + shoutIndex, debugVerbosity)
						ApplyWardDamage(currentWardPower, ImWaShoutWardpowerDamageGV.GetValue(), akCaster)
					EndIf
				EndIf
			EndIf
		EndIf
		
		If (aiStatus == 2) && (wardBlockingAngle == 180)
			wardCasterActor.DispelSpell(masterWardSpell)
			Debug.TraceConditional(self + "Dispelling master ward.", debugVerbosity)
		EndIf
		
		Debug.TraceConditional(self + "End of OnWardHit event." + " Time:" + Utility.GetCurrentRealTime() + "\n\n", debugVerbosity)	
		onWardHitFunctionBusy = False
		
	EndEvent

EndState


